@@ -37,15 +37,4 @@ int decrypt_cookie(ProtobufCAllocator *pa, gnutls_datum_t *key,
 			uint8_t *cookie, unsigned cookie_size, 
 			Cookie **msg);
 
-void cookie_db_init(void *pool, struct cookie_entry_db_st* db);
-void cookie_db_deinit(struct cookie_entry_db_st* db);
-void expire_cookies(struct cookie_entry_db_st* db);
-struct cookie_entry_st *new_cookie_entry(struct cookie_entry_db_st* db, proc_st *proc, void *cookie, unsigned cookie_size);
-struct cookie_entry_st *find_cookie_entry(struct cookie_entry_db_st* db, void *cookie, unsigned cookie_len);
-
-inline static void revive_cookie(struct cookie_entry_st * e)
-{
-	e->expiration = -1;
-}
-
 #endif
