@@ -105,6 +105,12 @@ struct ip_lease_st t;
 
 void steal_ip_leases(struct proc_st* proc, struct proc_st *thief)
 {
+	/* here we reset the old tun device, and assign the old addresses
+	 * to a new device. We cannot re-use the old device because the
+	 * fd is only available to the worker process and not here (main)
+	 */
+	reset_tun(proc);
+
 	thief->ipv4 = talloc_move(thief, &proc->ipv4);
 	thief->ipv6 = talloc_move(thief, &proc->ipv6);
 }
@@ -541,11 +547,11 @@ char buf[128];
 	}
 	
 	if (proc->ipv4)
-		mslog(s, proc, LOG_INFO, "assigned IPv4: %s",
+		mslog(s, proc, LOG_DEBUG, "assigned IPv4: %s",
 			human_addr((void*)&proc->ipv4->rip, proc->ipv4->rip_len, buf, sizeof(buf)));
 
 	if (proc->ipv6)
-		mslog(s, proc, LOG_INFO, "assigned IPv6: %s",
+		mslog(s, proc, LOG_DEBUG, "assigned IPv6: %s",
 			human_addr((void*)&proc->ipv6->rip, proc->ipv6->rip_len, buf, sizeof(buf)));
 
 	return 0;
