@@ -79,6 +79,8 @@ enum {
 	PS_AUTH_INIT, /* worker has sent an auth init msg */
 	PS_AUTH_CONT, /* worker has sent an auth cont msg */
 	PS_AUTH_COMPLETED, /* successful authentication */
+	PS_AUTH_USER_TERM /* user has terminated the session: this state is only valid in sec-mod.
+	                   * The reason for this mode is to indicate the cookie invalidation. */
 };
 
 /* Each worker process maps to a unique proc_st structure.
@@ -245,6 +247,7 @@ void  mslog_hex(const main_server_st * s, const struct proc_st* proc,
 
 int open_tun(main_server_st* s, struct proc_st* proc);
 void close_tun(main_server_st* s, struct proc_st* proc);
+void reset_tun(struct proc_st* proc);
 int set_tun_mtu(main_server_st* s, struct proc_st * proc, unsigned mtu);
 
 int send_cookie_auth_reply(main_server_st* s, struct proc_st* proc,
