@@ -21,11 +21,6 @@
 #include <unistd.h>
 #ifdef __linux__
 # include <sys/prctl.h>
-# if defined(ENABLE_LINUX_NS)
-#  include <sched.h>
-#  include <linux/sched.h>
-#  include <sys/syscall.h>
-# endif
 #endif
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -54,23 +49,6 @@ void pr_set_undumpable(const char *mod)
 #endif
 }
 
-#if defined(__linux__) && defined(ENABLE_LINUX_NS)
-pid_t safe_fork(void)
-{
-	long ret;
-	/* fork: 100%
-	 * CLONE_NEWPID|CLONE_NEWNET|CLONE_NEWIPC: 3%
-	 * CLONE_NEWPID|CLONE_NEWIPC: 27%
-	 * CLONE_NEWPID: 36%
-	 */
-	int flags = SIGCHLD|CLONE_NEWPID|CLONE_NEWIPC;
-	ret = syscall(SYS_clone, flags, 0, 0, 0);
-	if (ret == 0 && syscall(SYS_getpid)!= 1)
-		return -1;
-	return ret;
-}
-#endif
-
 SIGHANDLER_T ocsignal(int signum, SIGHANDLER_T handler)
 {
 	struct sigaction new_action, old_action;
@@ -114,8 +92,8 @@ int check_upeer_id(const char *mod, int debug, int cfd, uid_t uid, uid_t gid, ui
 	if (ruid)
 		*ruid = cr.uid;
 
-	if (debug >= 3 && cr.uid != 0 && (cr.uid != uid || cr.gid != gid)) {
-		syslog(LOG_DEBUG,
+	if (cr.uid != 0 && (cr.uid != uid || cr.gid != gid)) {
+		syslog(LOG_ERR,
 		       "%s: received unauthorized request from pid %u and uid %u",
 		       mod, (unsigned)cr.pid, (unsigned)cr.uid);
 		       return -1;
@@ -140,8 +118,8 @@ int check_upeer_id(const char *mod, int debug, int cfd, uid_t uid, uid_t gid, ui
 		syslog(LOG_DEBUG,
 		       "%s: received request from a processes with uid %u",
 		       mod, (unsigned)euid);
-	if (debug >= 3 && euid != 0 && (euid != uid || egid != gid)) {
-		syslog(LOG_DEBUG,
+	if (euid != 0 && (euid != uid || egid != gid)) {
+		syslog(LOG_ERR,
 		       "%s: received unauthorized request from a process with uid %u",
 			mod, (unsigned)euid);
 			return -1;
